--[[ 
A guy gets on a bus and starts threatening everybody: "I'll integrate you! I'll differentiate you!" So everybody gets scared and runs away, only one person stays. The guy comes up to him and says: "Aren't you scared, I'll integrate you, I'll differentiate you!" And the other guy says: "No, I am not scared, I am e^x."
--]]

local ADDON_NAME	= "Click4Info"

local DEBUG_MODE 	= true
local MAX_SEARCH_RESULTS = 250
local VERSION_CODE	= 2.7
local ROW_TYPE_ID 	= 1
local FREE_MOUSE	= false

--===================================--
--====== ScrollList Categories ======--
--===================================--
CLICK4INFO_CATEGORY_ALL 		= 1
CLICK4INFO_CATEGORY_FAVORITES 	= 2
CLICK4INFO_CATEGORY_SCRIPTS 	= 3
CLICK4INFO_CATEGORY_FUNCTIONS 	= 4
CLICK4INFO_CATEGORY_CONSTANTS 	= 5
CLICK4INFO_CATEGORY_VALUE 		= 6
CLICK4INFO_NUM_CATEGORIES		= 6

local SORTTYPE_NAME		= 1
local SORTTYPE_VALUE	= 2

Click4Info = ZO_Object:Subclass()


--===================================--
--======== Random Functions =========--
--===================================--
local function debugMsg(msg)
	if not DEBUG_MODE then return end
	
	d(msg)
end

local function CameraChanged()
	if not IsPlayerActivated() then return end
	
	if FREE_MOUSE then
		SetGameCameraUIMode(FREE_MOUSE)
	end
end

function Click4Info_Toggle_FreeMouse()
	FREE_MOUSE = not FREE_MOUSE
	SetGameCameraUIMode(FREE_MOUSE)
end


local function InitializeMenuBar()
	local menuBar = CLICK4INFO.menuBar
	ZO_MenuBar_ClearButtons(menuBar)
	
	for _, filterInfo in ipairs(CLICK4INFO.filterData) do
		local btn = ZO_MenuBar_AddButton(menuBar, filterInfo)
	end
	
	local menuBarObject = CLICK4INFO.menuBar.m_object
	local btn = menuBarObject:GetButtonControl(CLICK4INFO_CATEGORY_FAVORITES)
	
	ZO_MenuBarButtonTemplate_OnMouseUp(btn, 1, true)
	Click4Info_ChangePanel(btn, 1)
end

--===================================--
--======== ScrollIst Functions =========--
--===================================--
local function GetSortByType()
	local selectedCategory = CLICK4INFO.menuBar.m_object:GetSelectedDescriptor()
	
	if selectedCategory == CLICK4INFO_CATEGORY_CONSTANTS then
		if CLICK4INFO.constSort:GetState() == BSTATE_PRESSED then
			return SORTTYPE_VALUE
		end
	end
	return SORTTYPE_NAME
end
function Click4Info:SortDataTable(dataTable, sortByType)
	local sortType = sortByType or GetSortByType()
	local scrollList = self.scrollList
		
	if sortType == SORTTYPE_NAME then
		table.sort(dataTable, function(objA, objB)
			if objA.data.name < objB.data.name then return true end
			return false
		end)
		
	elseif sortType == SORTTYPE_VALUE then
		table.sort(dataTable, function(objA, objB)
			local objAVal = objA.data.value
			local objBVal = objB.data.value
			local objTypeA = type(objAVal)
			local objTypeB = type(objBVal)
			
			-- Required to handle the sort for the constant search
			-- Constant search can return objects of type 
			-- "string" or "number", if the types do not match
			-- they are sorted by type instead of value
			if objTypeA == objTypeB then
				return objAVal < objBVal
			else
				return (objTypeA < objTypeB)
			end
		end)
	end
	return dataTable
end

-- RowControl callback
local function setupDataRow(rowControl, data, scrollList)
	local label = rowControl:GetNamedChild("Label")
	local selectedCategory = CLICK4INFO.menuBar.m_object:GetSelectedDescriptor()
	local name = data.name
	
	if selectedCategory == CLICK4INFO_CATEGORY_CONSTANTS or selectedCategory == CLICK4INFO_CATEGORY_VALUE then
		if CLICK4INFO.showValue:GetState() == BSTATE_PRESSED then    
			local pattern = "<<1>> = <<2>>"
			name = zo_strformat(pattern, data.name, data.value)
		end
	end
	
	rowControl:SetText(name)
	rowControl:SetColor(1, 1, 1, 1)
	rowControl:SetSelected(ZO_ScrollList_IsDataSelected(scrollList, data))
end
	
local function OnRowSelection(previouslySelectedData, selectedData, reselectingDuringRebuild)
	local scrollList = CLICK4INFO.scrollList
	local selectedCategory = CLICK4INFO.menuBar.m_object:GetSelectedDescriptor()
	
	if selectedCategory == CLICK4INFO_CATEGORY_SCRIPTS then
		if selectedData then
			CLICK4INFO.notebookWin:SetHidden(false)
		else
			CLICK4INFO.notebookWin:SetHidden(true)
		end
	elseif selectedCategory == CLICK4INFO_CATEGORY_ALL or selectedCategory == CLICK4INFO_CATEGORY_FAVORITES then
		if not selectedData then
			local zgooWin = GetControl("ZgooFrame")
			if zgooWin ~= nil then
				zgooWin:SetHidden(true)
			end
		end
	end
end 

local function InitializeTooltipHandlers()
	local self = CLICK4INFO
	
	self.caseSens:SetHandler("OnMouseEnter", function(self)
		InitializeTooltip(InformationTooltip, CLICK4INFO.optionsWin, BOTTOM, 0, 0, TOP)
		SetTooltipText(InformationTooltip, "Use Case Sensitive Searches?")
	end)
	self.caseSens:SetHandler("OnMouseExit", function(self)
		ClearTooltip(InformationTooltip)
	end)
	
	self.constSort:SetHandler("OnMouseEnter", function(self)
		InitializeTooltip(InformationTooltip, CLICK4INFO.optionsWin, BOTTOM, 0, 0, TOP)
		SetTooltipText(InformationTooltip, "Sort Constant Searches Alphabetically or by Value?")
	end)
	self.constSort:SetHandler("OnMouseExit", function(self)
		ClearTooltip(InformationTooltip)
	end)
	
	self.showValue:SetHandler("OnMouseEnter", function(self)
		InitializeTooltip(InformationTooltip, CLICK4INFO.optionsWin, BOTTOM, 0, 0, TOP)
		SetTooltipText(InformationTooltip, "Show Constant Values in Search List?")
	end)
	self.showValue:SetHandler("OnMouseExit", function(self)
		ClearTooltip(InformationTooltip)
	end)
end

local function GetFavorites()
	local slashCommands = CLICK4INFO.sv.savedCommands[CLICK4INFO_CATEGORY_ALL]
	local favorites = {}
	
	for k,data in pairs(slashCommands) do
		if data.favorite then
			table.insert(favorites, data)
		end
	end
	return favorites
end
function Click4Info:GetCategoryIdTable(categoryId)
	if categoryId == CLICK4INFO_CATEGORY_FAVORITES then
		return GetFavorites()
	end
	return CLICK4INFO.sv.savedCommands[categoryId]
end
function Click4Info:UpdateSelectedCategory()
	local selectedCategory 	= self.menuBar.m_object:GetSelectedDescriptor()
	local sourceTable = self:GetCategoryIdTable(selectedCategory)
	
	self:UpdateScrollList(sourceTable)
end
function Click4Info:UpdateScrollListByCategoryId(categoryId)
	local dataTable = self:GetCategoryIdTable(categoryId)
	
	self:UpdateScrollList(dataTable)
end
function Click4Info:UpdateScrollList(sourceTable)
	local scrollList 	= self.scrollList
	local dataList 		= ZO_ScrollList_GetDataList(scrollList)
	ZO_ScrollList_Clear(scrollList)
	
	local next = next
	if not sourceTable or next(sourceTable) == nil then
		ZO_ScrollList_Commit(scrollList)
		return
	end
	local sourceCopy = {}
	
	--ZO_DeepTableCopy(sourceTable, sourceCopy)
	ZO_ShallowTableCopy(sourceTable, sourceCopy) --no meta data copy needed!
	
	
	for key, rowData in pairs(sourceCopy) do
		local entry = ZO_ScrollList_CreateDataEntry(ROW_TYPE_ID, rowData, rowData.categoryId)
		table.insert(dataList, entry)
	end
	
	self:SortDataTable(dataList)
	
	ZO_ScrollList_Commit(scrollList)
end

local function InitializeBindings()
	ZO_CreateStringId("SI_BINDING_NAME_CLICK4INFO_FREE_MOUSE", "Free Mouse")
	ZO_CreateStringId("SI_BINDING_NAME_CLICK4INFO_ZGOO_MOUSE", "/zgoo Mouse")
	ZO_CreateStringId("SI_BINDING_NAME_CLICK4INFO_RELOADUI", "ReloadUI")
	ZO_CreateStringId("SI_BINDING_NAME_CLICK4INFO_TOGGLE_WINDOW", "Toggle Window")
	ZO_CreateStringId("SI_BINDING_NAME_CLICK4INFO_PREV_CATEGORY", "Prev Category")
	ZO_CreateStringId("SI_BINDING_NAME_CLICK4INFO_NEXT_CATEGORY", "Next Category")
end
local function AreDataSetsEqual(data1, data2)
	if data1.name == data2. name and data1.categoryId == data2.categoryId then
		return true
	end
	return false
end
local function InitializeScrollList()
	local BUTTON_HEIGHT = 30
	local scrollList 	= CLICK4INFO.scrollList
	local dataList 		= ZO_ScrollList_GetDataList(scrollList)
	local dataTable		= sourceTable
	local cmds			= CLICK4INFO.sv.savedCommands
	
	ZO_ScrollList_Clear(scrollList)
	ZO_ScrollList_Commit(scrollList)
	
	ZO_ScrollList_AddDataType(scrollList, ROW_TYPE_ID, "Click4InfoRowControl", BUTTON_HEIGHT, setupDataRow)
	ZO_ScrollList_EnableSelection(scrollList, "ZO_ThinListHighlight", OnRowSelection) 
	--ZO_ScrollList_SetEqualityFunction(scrollList, ROW_TYPE_ID, AreDataSetsEqual)
	
	ZO_ScrollList_AddCategory(scrollList, CLICK4INFO_CATEGORY_ALL, parentId)
	CLICK4INFO:UpdateSelectedCategory()
end

local function InitializeOptions()
	local sortBtn = CLICK4INFO.constSort
	local caseSensBtn = CLICK4INFO.caseSens
	local showValueBtn = CLICK4INFO.showValue
	
	sortBtn.checkedText = "Value"
	sortBtn.uncheckedText = "Alphabetical"
	
	if CLICK4INFO.sv["CASE_SENS"] then
		ZO_CheckButton_SetChecked(caseSensBtn)
	else
        ZO_CheckButton_SetUnchecked(caseSensBtn)
	end
	if CLICK4INFO.sv["SORT_BY_VALUE"] then
		ZO_CheckButton_SetChecked(sortBtn)
	else
        ZO_CheckButton_SetUnchecked(sortBtn)
	end
	if CLICK4INFO.sv["SHOW_VALUE"] then
		ZO_CheckButton_SetChecked(showValueBtn)
	else
        ZO_CheckButton_SetUnchecked(showValueBtn)
	end
end

local function InitializeWindowPositions()
	local mainWinOffsetX 	= CLICK4INFO.sv.mainWinSettings.offsetX
	local mainWinOffsetY 	= CLICK4INFO.sv.mainWinSettings.offsetY
	local mainWinWidth 		= CLICK4INFO.sv.mainWinSettings.width
	local mainWinHeight		= CLICK4INFO.sv.mainWinSettings.height
	local notebookWinWidth 	= CLICK4INFO.sv.notebookWinSettings.width
	local notebookWinHeight	= CLICK4INFO.sv.notebookWinSettings.height
	
	CLICK4INFO.mainWin:ClearAnchors()
	CLICK4INFO.mainWin:SetAnchor(TOPLEFT, GuiRoot, TOPLEFT, mainWinOffsetX, mainWinOffsetY)
	CLICK4INFO.mainWin:SetDimensions(mainWinWidth, mainWinHeight)
	CLICK4INFO.mainWin:SetHidden(CLICK4INFO.sv["HIDE_WINDOW"])
	
	CLICK4INFO.notebookWin:SetDimensions(notebookWinWidth, notebookWinHeight)
end

-- Setup WAYPOINTIT, create all constants.
function Click4Info:New()
    local obj = ZO_Object.New(self)
	
	obj.name 			= ADDON_NAME
	obj.CodeVersion		= VERSION_CODE
	
--===========================================--
--======== DO NOT EVER CHANGE THIS ==========--
--===========================================--
	obj.SavedVarVersion = 2.0
--===========================================--
--===========================================--

	-----------------------------------------------------
	-- Colors
	obj.red			= "|cFF0000"
	obj.darkOrange	= "|cFFA500"
	obj.teal		= "|c00FFFF"
	obj.yellow		= "|cFFFF00" 
	
	-----------------------------------------------------
	-- Bind windows to obj
	-- ScrollList & Row Controls
	obj.mainWin 		= Click4InfoWin
	obj.scrollList		= Click4InfoWinScrollList
	obj.menuBar			= Click4InfoWinMenuBar
	obj.searchBox		= Click4InfoWinEditBox
	
	-- Options window
	obj.optionsWin		= Click4InfoOptions
	obj.caseSens		= Click4InfoOptionsCaseSens
	obj.constSort		= Click4InfoOptionsSortBtn
	obj.showValue		= Click4InfoOptionsShowValue
	
	-- Slash Command window controls
	obj.slashCmdWin				= Click4InfoSlashCmdWin
	obj.slashCmdNameEditBox		= Click4InfoSlashCmdWinNameEditBox
	obj.slashCmdEditBox			= Click4InfoSlashCmdWinCommandEditBox
	
	-- Script Window controls
	obj.notebookWin 			= Click4InfoNotebookWin
	obj.notebookNameEditBox 	= Click4InfoNotebookWinScriptNameEditBox
	obj.notebookScriptEditBox 	= Click4InfoNotebookWinScriptEditBox
	-----------------------------------------------------
	
    return obj
end

-- Initialize everything else
function Click4Info:Initialize()
	local defaultSavedVars = {
		savedCommands = {
			[CLICK4INFO_CATEGORY_ALL] 			= {},
			[CLICK4INFO_CATEGORY_FAVORITES] 	= {},
			[CLICK4INFO_CATEGORY_SCRIPTS] 		= {},
			[CLICK4INFO_CATEGORY_FUNCTIONS] 	= {},
			[CLICK4INFO_CATEGORY_CONSTANTS] 	= {},
			[CLICK4INFO_CATEGORY_VALUE] 		= {},
		},
		["CASE_SENS"] = false,
		["SORT_BY_VALUE"] = false,
		["SHOW_VALUE"] = false,
		
		["HIDE_WINDOW"] = false,
		
		mainWinSettings = {
			offsetX	= 100,
			offsetY = 100,
			width	= 350,
			height	= 450,
		},
		notebookWinSettings = {
			width	= 600,
			height	= 600,
		},
	}
	self.sv = ZO_SavedVars:NewAccountWide("Click4InfoSavedVars", self.SavedVarVersion, nil, defaultSavedVars)

	InitializeBindings()
	InitializeScrollList()
	InitializeMenuBar()
	InitializeOptions()
	InitializeTooltipHandlers()
	InitializeWindowPositions()
	
	
	EVENT_MANAGER:RegisterForEvent(ADDON_NAME, EVENT_GAME_CAMERA_DEACTIVATED, CameraChanged)
	EVENT_MANAGER:RegisterForEvent(ADDON_NAME, EVENT_GAME_CAMERA_UI_MODE_CHANGED, CameraChanged)
end

-------------------------------------------------------------------
--  OnAddOnLoaded  --
-------------------------------------------------------------------
local function OnAddOnLoaded(event, addonName)
	if addonName == ADDON_NAME then
		CLICK4INFO = Click4Info:New()
		CLICK4INFO:Initialize()
	end
end
-------------------------------------------------------------------
-------------------------------------------------------------------
local function OnPlayerDeactivation(eventCode)
-- Clear these out, no need to save them.
	CLICK4INFO.sv.savedCommands[CLICK4INFO_CATEGORY_FUNCTIONS]	= {}
	CLICK4INFO.sv.savedCommands[CLICK4INFO_CATEGORY_CONSTANTS]	= {}
	CLICK4INFO.sv.savedCommands[CLICK4INFO_CATEGORY_VALUE]		= {}
end
---------------------------------------------------------------------
--  Register Events --
---------------------------------------------------------------------
EVENT_MANAGER:RegisterForEvent(ADDON_NAME, EVENT_ADD_ON_LOADED, OnAddOnLoaded)
EVENT_MANAGER:RegisterForEvent(ADDON_NAME, EVENT_PLAYER_DEACTIVATED, OnPlayerDeactivation)


