

-----------------------------------------------------------
-----------------------------------------------------------
-- Slash Commands --
-----------------------------------------------------------
-----------------------------------------------------------
local function Trim(s)
	if s == nil or s == "" then return s end
	return s:find'^%s*$' and '' or s:match'^%s*(.*%S)'
end

local function AddSlashCommand(arguments)
	local tArguments = {}
	if arguments == nil or arguments == "" then return end
	local cmd, cmdName = string.match(arguments, "(.+),,(.+)")

	if ((cmd == nil) or (cmdName == nil)) then
		d(CLICK4INFO.red.."Enter Command as <script>,, <name>")
		d(CLICK4INFO.red.."Make sure your using two commas ,,")
		return
	end

	cmd = Trim(cmd)
	cmdName = Trim(cmdName)

	local commandTable = CLICK4INFO.sv.savedCommands[CLICK4INFO_CATEGORY_ALL]

	if not CLICK4INFO:DoesSlashCommandExist(cmdName) then
		table.insert(commandTable, {["name"]=cmdName, ["value"]=cmd, ["categoryId"]=CLICK4INFO_CATEGORY_ALL})
	else
		ZO_Alert(UI_ALERT_CATEGORY_ERROR, SOUNDS.NEGATIVE_CLICK, CLICK4INFO.darkOrange.."Script Name already exists. Pick a new name or select that script to edit it.")
	end
	-- Only need to refresh scrollList on the all panel
	local selectedCategory = CLICK4INFO.menuBar.m_object:GetSelectedDescriptor()
	if selectedCategory == CLICK4INFO_CATEGORY_ALL then
		CLICK4INFO:UpdateSelectedCategory()
	end
end

local function DoSlashSearch(searchText, categoryId)
--[[ If I do this, it wont update the list, before I call the search,
that might be better 
	local menuBarObject = CLICK4INFO.menuBar.m_object
    local buttonObject 	= menuBarObject:ButtonObjectForDescriptor(btnDescriptor)
	menuBarObject:SetClickedButton(buttonObject, true)
--]]	
	local menuBarObject = CLICK4INFO.menuBar.m_object
	local btn = menuBarObject:GetButtonControl(categoryId)
	ZO_MenuBarButtonTemplate_OnMouseUp(btn, 1, true)
	
	Click4Info_Search(searchText)
end

local function SlashCommandSearch(searchText)
	DoSlashSearch(searchText, CLICK4INFO_CATEGORY_ALL)
end
local function FavoriteSearch(searchText)
	DoSlashSearch(searchText, CLICK4INFO_CATEGORY_FAVORITES)
end
local function ScriptSearch(searchText)
	DoSlashSearch(searchText, CLICK4INFO_CATEGORY_SCRIPTS)
end
local function FunctionSearch(searchText)
	DoSlashSearch(searchText, CLICK4INFO_CATEGORY_FUNCTIONS)
end
local function ConstantSearch(searchText)
	DoSlashSearch(searchText, CLICK4INFO_CATEGORY_CONSTANTS)
end
local function ValueSearch(searchText)
	DoSlashSearch(searchText, CLICK4INFO_CATEGORY_VALUE)
end


SLASH_COMMANDS["/slashadd"] 	= AddSlashCommand
SLASH_COMMANDS["/slasha"] 		= AddSlashCommand
SLASH_COMMANDS["/clicka"] 		= AddSlashCommand

SLASH_COMMANDS["/slashs"] 	= SlashCommandSearch
SLASH_COMMANDS["/favs"] 	= FavoriteSearch
SLASH_COMMANDS["/scripts"] 	= ScriptSearch

SLASH_COMMANDS["/funs"] 	= FunctionSearch
SLASH_COMMANDS["/cons"] 	= ConstantSearch
SLASH_COMMANDS["/vals"] 	= ValueSearch


-- Thanks to Garkin for posting how to change languages on the forums --
SLASH_COMMANDS["/langen"] = function() SetCVar("language.2", "en") end
SLASH_COMMANDS["/langde"] = function() SetCVar("language.2", "de") end
SLASH_COMMANDS["/langfr"] = function() SetCVar("language.2", "fr") end

SLASH_COMMANDS["/c4i"] = function() Click4Info_Toggle_Window() end
SLASH_COMMANDS["/click4info"] = function() Click4Info_Toggle_Window() end


