


--=====================================--
--======== LOCAL/OTHER  ==========--
--=====================================--
local function SetPanelCategory(categoryId)
	local menuBarObject = CLICK4INFO.menuBar.m_object
	local btn 			= menuBarObject:GetButtonControl(categoryId)
	
	ZO_MenuBarButtonTemplate_OnMouseUp(btn, 1, true)
	Click4Info_ChangePanel(btn, 1)
end

-- Used to update the favorite flag for the data saved in saved vars
local function UpdateSvScript(selectedData, newData)
	local savedCatTable = CLICK4INFO.sv.savedCommands[CLICK4INFO_CATEGORY_SCRIPTS]
	
	for key, savedData in pairs(savedCatTable) do
		if savedData.name == selectedData.name 
		and savedData.value == selectedData.value then
			-- Update the name & value
			savedData.name = newData.name 
			savedData.value = newData.value 
			return
		end
	end
end
local function ClearNotebook()
	local nameEditBox 	= CLICK4INFO.notebookNameEditBox
	local scriptEditBox = CLICK4INFO.notebookScriptEditBox
	nameEditBox:Clear()
	scriptEditBox:Clear()
	nameEditBox:TakeFocus()
	
	CLICK4INFO.notebookWin.currentScrollListData = nil
end
--=====================================--
--======== FAVORITES  ==========--
--=====================================--
-- Used to update the favorite flag for the data saved in saved vars
local function UpdateSvFavoriteStatus(data, isFavorite)
	local savedCatTable = CLICK4INFO.sv.savedCommands[data.categoryId]
	
	for key, savedData in pairs(savedCatTable) do
		if savedData.name == data.name and savedData.value == data.value then
			savedData.favorite = isFavorite
			return
		end
	end
end

local function MarkFavorite(data, bIsFavorite)
	local selectedCategory = CLICK4INFO.menuBar.m_object:GetSelectedDescriptor()
	-- Favorites only allowed in these categories:
	if not (selectedCategory == CLICK4INFO_CATEGORY_ALL or selectedCategory == CLICK4INFO_CATEGORY_FAVORITES) then
		return
	end
	
	-- Set the scrollList data favorite flag
	data.favorite = bIsFavorite
	-- Update the saved var favorite flag
	UpdateSvFavoriteStatus(data, bIsFavorite)
	
	if selectedCategory == CLICK4INFO_CATEGORY_FAVORITES then
		CLICK4INFO:UpdateSelectedCategory()
	end
end


--=====================================--
--======== SLASH COMMAND FUNCTIONS	 ==========--
--=====================================--
local function DeleteSlashCommand(data)
	-- we can only delete slash commands here
	if data.categoryId ~= CLICK4INFO_CATEGORY_ALL then return end
	
	local savedCatTable = CLICK4INFO.sv.savedCommands[CLICK4INFO_CATEGORY_ALL]
	
	-- Remove it from the saved vars
	for k, v in pairs(savedCatTable) do
		if v.name == data.name then
			table.remove(savedCatTable, k)
			break
		end
	end
	-- Remove it from the scrollList
	CLICK4INFO:UpdateSelectedCategory()
end
local function RunSlashCommand(command)
	if not command or command == "" then return end
	
	if string.find(command, "^/") then
		CHAT_SYSTEM:StartTextEntry(command)
		CHAT_SYSTEM:SubmitTextEntry()
	else
		ZO_Alert(UI_ALERT_CATEGORY_ERROR, SOUNDS.NEGATIVE_CLICK, CLICK4INFO.darkOrange.."Invalid slash command")
	end
end
function Click4Info_Toggle_AddSlashCmdWin(self, button, upInside)
	if not (upInside and button == 1) then return end
	local bHide = not CLICK4INFO.slashCmdWin:IsHidden()
	CLICK4INFO.slashCmdWin:SetHidden(bHide)
end
function Click4Info:DoesSlashCommandExist(slashCmdName)
	local slashCommands = CLICK4INFO.sv.savedCommands[CLICK4INFO_CATEGORY_ALL]
	for key, data in pairs(slashCommands) do
		if data.name == slashCmdName then
			return true
		end
	end
	return false
end
function Click4Info_AddSlashCommand(self, button, upInside)
	if not upInside then return end
	local slashCmdName = CLICK4INFO.slashCmdNameEditBox:GetText()
	local slashCmd = CLICK4INFO.slashCmdEditBox:GetText()
	
	if not string.match(slashCmd, "^/") then
		slashCmd = "/"..slashCmd
	end
	if not slashCmdName or slashCmdName == "" then 
		ZO_Alert(UI_ALERT_CATEGORY_ERROR, SOUNDS.NEGATIVE_CLICK, CLICK4INFO.darkOrange.."You must enter a name for your command.")
		return 
	end
	if not slashCmd or slashCmd == "" or slashCmd == "/" then
		ZO_Alert(UI_ALERT_CATEGORY_ERROR, SOUNDS.NEGATIVE_CLICK, CLICK4INFO.darkOrange.."You must enter a command.")
		return 
	end
	
	-- Add a check to see if it already exists
	if not CLICK4INFO:DoesSlashCommandExist(slashCmdName) then
		local rowData = {["name"] = slashCmdName, ["value"] = slashCmd, ["categoryId"] = CLICK4INFO_CATEGORY_ALL}
		table.insert(CLICK4INFO.sv.savedCommands[CLICK4INFO_CATEGORY_ALL], rowData)
		local selectedCategory = CLICK4INFO.menuBar.m_object:GetSelectedDescriptor()
		if (selectedCategory == CLICK4INFO_CATEGORY_ALL 
		or selectedCategory == CLICK4INFO_CATEGORY_FAVORITES) then
			CLICK4INFO:UpdateSelectedCategory()
		end
	else
		ZO_Alert(UI_ALERT_CATEGORY_ERROR, SOUNDS.NEGATIVE_CLICK, CLICK4INFO.darkOrange.."Slash command name already exists.")
	end
end

--=====================================--
--======== SCRIPT FUNCTIONS	 ==========--
--=====================================--

--[[ 
What if you delete the script you are on?
Do I need to update
	CLICK4INFO.notebookWin.currentScrollListData = data
and clear the edit boxes?
--]]



local function DeleteScript(data)
	if data.categoryId 		~= CLICK4INFO_CATEGORY_SCRIPTS then return end
	local savedCommands		= CLICK4INFO.sv.savedCommands
	local savedCatTable 	= savedCommands[CLICK4INFO_CATEGORY_SCRIPTS]
	
	-- If the script we deleted was the selected script, 
	-- Clear the notebook
	local function TryClearNotebook(data)
		local scrollList = CLICK4INFO.scrollList
		local selectedData = ZO_ScrollList_GetSelectedData(scrollList)
		
		if selectedData and selectedData.name == data.name 
		and selectedData.value == data.value then
			ClearNotebook()
		end
	end
	
	-- Remove it from the saved vars
	for k, tableData in pairs(savedCatTable) do
		if tableData.name == data.name then
			table.remove(savedCatTable, k)
			TryClearNotebook(data)
			break
		end
	end
	CLICK4INFO:UpdateSelectedCategory()
end
local function RunScript(command)
	if not command or command == "" then return end
	
	local script = zo_loadstring(command)
	if script then
		script()
	end
end
local function ShowScript(data)
	local scriptName 	= data.name
	local script 		= data.value
	
	CLICK4INFO.notebookNameEditBox:SetText(scriptName)
	CLICK4INFO.notebookScriptEditBox:SetText(script)
	
	-- Used for identifying the data in the scrollList after
	-- the user makes changes to the scripName or script
	CLICK4INFO.notebookWin.currentScrollListData = data
end
function Click4Info_Toggle_ScriptWindow(self, button, upInside)
	if not (upInside and button == 1) then return end
	local notebookWin = CLICK4INFO.notebookWin
	
	notebookWin:SetHidden(not notebookWin:IsHidden())
end
function Click4Info_Toggle_Options(self, button, upInside)
	if not (upInside and button == 1) then return end
	local optionsWin = CLICK4INFO.optionsWin
	
	optionsWin:SetHidden(not optionsWin:IsHidden())
end
function Click4Info_New_Lua_Page(self, button, upInside)
	if not upInside then return end
	local editBox = self:GetParent():GetNamedChild("ScriptEditBox")
	local scriptNameEditBox = self:GetParent():GetNamedChild("ScriptNameEditBox")
	local scrollList = CLICK4INFO.scrollList
	
	-- Clear the edit Boxes & give focus to the name box
	editBox:Clear()
	scriptNameEditBox:Clear()
	scriptNameEditBox:TakeFocus()
	
	-- Clear the current data link to the scroll List data
	-- If its a new page, its not in the scroll List
	CLICK4INFO.notebookWin.currentScrollListData = nil
	
	if(scrollList.selectedData) then
		scrollList.selectedData = nil
		scrollList.selectedDataIndex = nil
		if(scrollList.selectedControl) then
			local animation = scrollList.selectedControl["SelectionAnimation"]
			if animation then
				animation:PlayBackward()
			end
			scrollList.selectedControl = nil
		end
	end
end

function Click4Info_Run_Code(self, button, upInside)
	local code = self:GetParent():GetNamedChild("ScriptEditBox"):GetText()
	RunScript(code)
end

local function DoesScriptExist(scriptName)
	if not scriptName or scriptName == "" then return true end
	local scriptsTable = CLICK4INFO.sv.savedCommands[CLICK4INFO_CATEGORY_SCRIPTS]
	
	for key, data in pairs(scriptsTable) do
		if data.name == scriptName then
			return data
		end
	end
end
local function SelectCurrentScript()
	local scrollListData 	= CLICK4INFO.notebookWin.currentScrollListData
	if not scrollListData then return end
	
	local scriptName 	= scrollListData.name
	local scriptValue	= scrollListData.value
	local scrollList 	= CLICK4INFO.scrollList
	
	for k,dataEntry in pairs(scrollList.data) do
		local data = dataEntry.data
		
		if data.name == scriptName and data.value == scriptValue then
			ZO_ScrollList_SelectData(scrollList, data)
			return
		end
	end
	ZO_ScrollList_SelectData(scrollList, nil)
end

local function SelectCurrentScript()
	local scrollListData 	= CLICK4INFO.notebookWin.currentScrollListData
	if not scrollListData then return end
	
	local scriptName 	= scrollListData.name
	local scriptValue	= scrollListData.value
	local scrollList 	= CLICK4INFO.scrollList
	
	for k,dataEntry in pairs(scrollList.data) do
		local data = dataEntry.data
		
		if data.name == scriptName and data.value == scriptValue then
			ZO_ScrollList_SelectData(scrollList, data)
			return
		end
	end
	ZO_ScrollList_SelectData(scrollList, nil)
end

function Click4Info_Save_Code(self, button, upInside)
	if not upInside then return end
	local newScriptName = CLICK4INFO.notebookNameEditBox:GetText()
	local newScript 	= CLICK4INFO.notebookScriptEditBox:GetText()
	local scrollList 	= CLICK4INFO.scrollList
	
	if not newScriptName or newScriptName == "" then 
		ZO_Alert(UI_ALERT_CATEGORY_ERROR, SOUNDS.NEGATIVE_CLICK, CLICK4INFO.darkOrange.."You must enter a name for your script.")
		return
	end
	if not newScript or newScript == "" then 
		ZO_Alert(UI_ALERT_CATEGORY_ERROR, SOUNDS.NEGATIVE_CLICK, CLICK4INFO.darkOrange.."You must enter a script.")
		return 
	end
	
	local menuBarObject 	= CLICK4INFO.menuBar.m_object
	local selectedCategory 	= menuBarObject:GetSelectedDescriptor()
	local scrollListData 	= CLICK4INFO.notebookWin.currentScrollListData
	local newData = {["name"] = newScriptName, ["value"] = newScript, ["categoryId"] = CLICK4INFO_CATEGORY_SCRIPTS}
	
	-- If needed switch to the scripts cateogry & select the script were
	-- working on
	if selectedCategory ~= CLICK4INFO_CATEGORY_SCRIPTS then
		SetPanelCategory(CLICK4INFO_CATEGORY_SCRIPTS)
	end
	SelectCurrentScript()
	local selectedData 		= ZO_ScrollList_GetSelectedData(scrollList)
	
	-- If the newScriptName already exists it should be selected
	if selectedData then
		-- SelectedData could be used, but it also changes in this if statement
		-- In case of code changes use scrollListData, safer & don't have to 
		-- worry about the order these events here are called in
		UpdateSvScript(scrollListData, newData)
		
		-- Update the script data saved in the scroll List
		selectedData.name = newScriptName
		selectedData.value = newScript
	
	-- If they were editing a script we already know if it exists or not
	-- This is for newly created scripts to check & see if a script with
	-- that name already exists
	elseif DoesScriptExist(newScriptName) then
		ZO_Alert(UI_ALERT_CATEGORY_ERROR, SOUNDS.NEGATIVE_CLICK, CLICK4INFO.darkOrange.."Script Name already exists. Pick a new name or select that script to edit it.")
	else
		-- else insert it into the table.
		table.insert(CLICK4INFO.sv.savedCommands[CLICK4INFO_CATEGORY_SCRIPTS], newData)
	end
	
	-- Update the scrollListData saved in the notebook window
	-- So we know which scroll list data item it belongs to when the 
	-- user changes the script name or script
	CLICK4INFO.notebookWin.currentScrollListData = newData
	
	-- Update the table to show the new script & select it
	CLICK4INFO:UpdateSelectedCategory()
	SelectCurrentScript()
end



--=====================================--
--======== XML CLICK FUNCTIONS	 ==========--
--=====================================--
function Click4Info_NextCategory()
	local selectedCategory = CLICK4INFO.menuBar.m_object:GetSelectedDescriptor()
	local newCategory = selectedCategory%CLICK4INFO_NUM_CATEGORIES + 1
	
	SetPanelCategory(newCategory)
end
function Click4Info_PrevCategory()
	local menuBarObject = CLICK4INFO.menuBar.m_object
	local newCategory = menuBarObject:GetSelectedDescriptor() - 1
	
	if newCategory == 0 then
		SetPanelCategory(CLICK4INFO_NUM_CATEGORIES)
		return
	end
	SetPanelCategory(newCategory)
end

function Click4Info_RowSelect(self, button, upInside)
	if not upInside then return end		
	local data = self.dataEntry.data
	local dataCategoryId = data.categoryId
	
	if button == 2 then
		if dataCategoryId == CLICK4INFO_CATEGORY_ALL then
			ClearMenu()
			if data.favorite then
				AddCustomMenuItem("Unmark as favorite", function() MarkFavorite(data, false) end)
			else
				AddCustomMenuItem("Mark as Favorite", function() MarkFavorite(data, true) end)
			end
			AddCustomMenuItem("Delete slash command", function() DeleteSlashCommand(data) end)
			ShowMenu()
		elseif dataCategoryId == CLICK4INFO_CATEGORY_SCRIPTS then
			ClearMenu()
			AddCustomMenuItem("Delete script", function() DeleteScript(data) end)
			ShowMenu()
		end
		
	elseif button == 1 then
		if dataCategoryId == CLICK4INFO_CATEGORY_ALL then
			RunSlashCommand(data.value)
		elseif dataCategoryId == CLICK4INFO_CATEGORY_SCRIPTS then
			ShowScript(data)
		end
		ZO_ScrollList_MouseClick(CLICK4INFO.scrollList, self)
	end
end
function Click4Info_OnRowMouseEnter(self)
	local command = self.dataEntry.data.value
	if type(command) ~= "string" and type(command) ~= "number" then return end
	if command == "" then return end
	
	InitializeTooltip(InformationTooltip, self, LEFT, 26, 0, RIGHT)
	InformationTooltip:AddLine(command)
end
function Click4Info_OnRowMouseExit(self)
	ClearTooltip(InformationTooltip)
end
function Click4Info_Toggle_Window()
	local bSetHidden = not CLICK4INFO.mainWin:IsHidden()
	CLICK4INFO.mainWin:SetHidden(bSetHidden)
	CLICK4INFO.sv["HIDE_WINDOW"] = bSetHidden
	
	if bSetHidden then
		CLICK4INFO.optionsWin:SetHidden(bSetHidden)
		CLICK4INFO.slashCmdWin:SetHidden(bSetHidden)
		CLICK4INFO.notebookWin:SetHidden(bSetHidden)
	end
end
function Click4Info_SetCaseSens(self, button)
	if button ~= 1 then return end
	if self:GetState() == BSTATE_PRESSED then
		CLICK4INFO.sv["CASE_SENS"] = true
		return
	end
	CLICK4INFO.sv["CASE_SENS"] = false
end
function Click4Info_SetShowValue(self, button)
	if button ~= 1 then return end
	if self:GetState() == BSTATE_PRESSED then
		CLICK4INFO.sv["SHOW_VALUE"] = true
		return
	end
	CLICK4INFO.sv["SHOW_VALUE"] = false
end
function Click4Info_SetSort(self, button)
	if button ~= 1 then return end
	if self:GetState() == BSTATE_PRESSED then
		CLICK4INFO.sv["SORT_BY_VALUE"] = true
		return
	end
	CLICK4INFO.sv["SORT_BY_VALUE"] = false
end

function Click4Info_ChangePanel(self, button)
	local selectedCategory 	= self.m_object:GetDescriptor()
	
	CLICK4INFO.searchBox:Clear()
	CLICK4INFO:UpdateSelectedCategory()
	
	-- If they selected the scripts category, select the current
	-- script in the notebook window...should we force it visible? Nah
	if selectedCategory == CLICK4INFO_CATEGORY_SCRIPTS then
		SelectCurrentScript()
	end
end


--=====================================--
--======== XML MOVE & RESIZE FUNCTIONS	 ==========--
--=====================================--
function Click4Info_OnResizeStart_MainWin(self)
	EVENT_MANAGER:RegisterForUpdate("Click4Info_Resize_MainWin", 100, function()
		CLICK4INFO:UpdateSelectedCategory() 
	end)
end
function Click4Info_OnResizeStop_MainWin(self)
	EVENT_MANAGER:UnregisterForUpdate("Click4Info_Resize_MainWin")
	
	CLICK4INFO.sv.mainWinSettings.width 	= self:GetWidth()
	CLICK4INFO.sv.mainWinSettings.height	= self:GetHeight()
end
function Click4Info_OnMoveStop_MainWin(self)
	CLICK4INFO.sv.mainWinSettings.offsetX	= self:GetLeft()
	CLICK4INFO.sv.mainWinSettings.offsetY	= self:GetTop()
end

function Click4Info_OnResizeStop_NotebookWin(self)
	CLICK4INFO.sv.notebookWinSettings.width 	= self:GetWidth()
	CLICK4INFO.sv.notebookWinSettings.height	= self:GetHeight()
end
	
	
	
	
	
	
	
	
	
	
	